/*
 * 2D Physic Engine
 * body.h:
 * Based on pikuma.com Learn Game Physics Engine Programming course.
 * Copyright (c) 2022 986-Studio. All rights reserved.
 *
 * Created by Manoël Trapier on 07/06/2022.
 */

#ifndef PHYSICENGINE_PHYSICS_BODY_H
#define PHYSICENGINE_PHYSICS_BODY_H

#include <vector>

#include <physics/vec2.h>
#include <physics/shape.h>

class body
{
public:
    vec2 position;
    vec2 acceleration;
    vec2 velocity;

    vec2 sum_of_forces;
    vec2 frame_sum;

    double mass;
    double invMass;

    shape *shp;

    /* Only used for debug display */
    std::vector<vec2> forces;

public:
    body(const shape &s, double x, double y, double mass) : position(x, y), mass(mass)
    {
        this->shp = s.clone();
        this->invMass = 0;
        if (mass != 0.0)
        {
            this->invMass = 1 / mass;
        }
    };

    ~body()
    {
        delete this->shp;
    };

    void clearForces()
    {
        this->sum_of_forces = vec2(0, 0);
    }

    void forceDebug(bool showVelocity, bool showSum, bool showAll);

    void addForce(const vec2 &force);
    void integrate(double dt);
};

#endif /* PHYSICENGINE_PHYSICS_BODY_H */
